package net.meutel.recettes.api.controller;

import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.hasSize;
import static org.mockito.Mockito.when;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.oauth2Login;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;

import net.meutel.recettes.api.exception.ItemNotFoundException;
import net.meutel.recettes.api.model.Receipe;
import net.meutel.recettes.api.service.ReceipeService;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@AutoConfigureMockMvc
public class ReceipeControllerTest {

        @Autowired
        private MockMvc mockMvc;

        @MockBean
        private ReceipeService service;
        
    @Test
    public void getReceipeById_noAuth_302() throws Exception {
        when(service.getReceipeById("TEST"))
                .thenThrow(new ItemNotFoundException("Receipe", "TEST"));

        this.mockMvc.perform(get("/receipes/TEST"))
                .andExpect(status().is3xxRedirection());
    }

    @Test
    public void getReceipeById_ItemNotFound_404() throws Exception {
        when(service.getReceipeById("TEST"))
                .thenThrow(new ItemNotFoundException("Receipe", "TEST"));

        this.mockMvc.perform(get("/receipes/TEST").with(oauth2Login()))
                .andExpect(status().isNotFound());
    }

        @Test
        public void getReceipeById_exists_success() throws Exception {
                Receipe r1 = new Receipe()
                                .name("pates carbo");
                when(service.getReceipeById("TEST"))
                                .thenReturn(r1);

                this.mockMvc.perform(get("/receipes/TEST").with(oauth2Login()))
                                .andExpect(status().isOk())
                                .andExpect(jsonPath("$.name", equalTo("pates carbo")));
        }

        @Test
        public void findReceipes_none_empty() throws Exception {
                this.mockMvc.perform(get("/receipes").with(oauth2Login()))
                                .andExpect(status().isOk())
                                .andExpect(jsonPath("$").isArray())
                                .andExpect(jsonPath("$").isEmpty());
        }

        @Test
        public void findReceipes_exists_list() throws Exception {
                Receipe r1 = new Receipe()
                                .name("pates carbo");
                Receipe r2 = new Receipe()
                                .name("soupe a l'oignon");
                when(service.findReceipes())
                                .thenReturn(List.of(r1, r2));

                this.mockMvc.perform(get("/receipes").with(oauth2Login()))
                                .andExpect(status().isOk())
                                .andExpect(jsonPath("$").isArray())
                                .andExpect(jsonPath("$", hasSize(2)))
                                .andExpect(jsonPath("$.[*].name").exists())
                                .andExpect(jsonPath("$.[0].name", equalTo("pates carbo")))
                                .andExpect(jsonPath("$.[*].description").doesNotExist());
        }
}
