package net.meutel.recettes.api.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import net.meutel.recettes.api.entity.QuantityEntity;
import net.meutel.recettes.api.entity.ReceipeEntity;
import net.meutel.recettes.api.entity.ReceipeIngredientEntity;
import net.meutel.recettes.api.entity.ReceipeStepEntity;
import net.meutel.recettes.api.entity.ReceipeYieldEntity;
import net.meutel.recettes.api.exception.ItemNotFoundException;
import net.meutel.recettes.api.model.Ingredient;
import net.meutel.recettes.api.model.Receipe;
import net.meutel.recettes.api.model.ReceipeReceipeYield;
import net.meutel.recettes.api.model.Step;
import net.meutel.recettes.api.repository.ReceipeRepository;

@SpringBootTest
public class ReceipeServiceImplTest {

    @Autowired
    ReceipeService tested;

    @MockBean
    ReceipeRepository repo;

    @Test
    void getReceipeById_noData_exception() {
        Assertions.assertThrows(ItemNotFoundException.class, () -> tested.getReceipeById("TEST"));
    }

    @Test
    void getReceipeById_exists_receipe() {
        var r1 = buildReceipe();

        when(repo.findById("TEST"))
                .thenReturn(Optional.of(r1));

        var result = tested.getReceipeById("TEST");

        assertThat(result)
                .isNotNull()
                .extracting(Receipe::getName)
                .isEqualTo("pates carbo");
        assertThat(result)
                .extracting(Receipe::getReceipeYield)
                .isNotNull()
                .extracting(ReceipeReceipeYield::getQuantity)
                .isNotNull();
        assertThat(result)
                .extracting(Receipe::getReceipeYield)
                .extracting(ReceipeReceipeYield::getOf)
                .isEqualTo("personne");
        assertThat(result)
                .extracting(Receipe::getIngredients)
                .isNotNull();
        assertThat(result.getIngredients())
                .singleElement()
                .extracting(Ingredient::getText)
                .isEqualTo("Parmesan");
        assertThat(result.getSteps())
                .isNotNull();
        assertThat(result.getSteps())
        .singleElement()
                .extracting(Step::getPosition, Step::getText, Step::getHint)
                .contains(1, "Faire bouillir de l'eau", "attention aux explosions");
    }

    @Test
    void findReceipes_noData_empty() {
        var result = tested.findReceipes();

        assertThat(result)
                .isNotNull()
                .isEmpty();
    }

    @Test
    void findReceipes_exists_list() {
        ReceipeEntity r1 = new ReceipeEntity();
        r1.setName("pates carbo");
        ReceipeEntity r2 = new ReceipeEntity();
        r2.setName("soupe a l'oignon");

        when(repo.findAll())
                .thenReturn(List.of(r1, r2));

        var result = tested.findReceipes();

        assertThat(result)
                .isNotNull()
                .hasSize(2);
        assertThat(result).element(0)
                .returns("pates carbo", Receipe::getName);
        assertThat(result).element(1)
                .returns("soupe a l'oignon", Receipe::getName);
    }

    private ReceipeEntity buildReceipe() {
        var r1 = new ReceipeEntity();
        r1.setName("pates carbo");
        var yield = new ReceipeYieldEntity();
        yield.setOf("personne");
        var qty = new QuantityEntity();
        qty.setValue(new BigDecimal(1));
        yield.setQuantity(qty);
        r1.setReceipeYield(yield);
        var i1 = new ReceipeIngredientEntity();
        var i1qty = new QuantityEntity();
        i1qty.setValue(new BigDecimal(100));
        i1qty.setUnit("g");
        i1.setQuantity(i1qty);
        i1.setText("Parmesan");
        r1.setIngredients(List.of(i1));
        var step1 = new ReceipeStepEntity();
        step1.setPosition(1);
        step1.setText("Faire bouillir de l'eau");
        step1.setHint("attention aux explosions");
        r1.setSteps(List.of(step1));
        return r1;
    }

}
